import socket

from zrok_api.api import EnvironmentApi
from zrok_api.models.enable_request import EnableRequest as ApiEnableRequest
from zrok_api.models.disable_request import DisableRequest as ApiDisableRequest
from zrok_api.exceptions import ApiException
from zrok2.environment.root import Root, Environment


def enable(root: Root, token: str, description: str = None, host: str = None) -> Environment:
    """Enable a zrok environment from an account token.

    Analogous to ``zrok2 enable <token>``. Safe to call when already enabled
    (returns existing environment without modification).

    Args:
        root: The Root environment context (from ``root.Load()``).
        token: Account enable token.
        description: Optional description for this environment.
        host: Optional hostname override (defaults to system hostname).

    Returns:
        The Environment dataclass populated with token, identity, and endpoint.
    """
    if root.IsEnabled():
        return root.env

    if host is None:
        host = socket.gethostname()

    if description is None:
        description = ""

    # Temporarily set the account token so Client() can authenticate
    api_endpoint = root.ApiEndpoint()
    root.env.Token = token
    root.env.ApiEndpoint = api_endpoint.endpoint

    try:
        zrok = root.Client()
    except Exception as e:
        root.env = Environment()
        raise Exception("error getting zrok client", e)

    try:
        req = ApiEnableRequest(description=description, host=host)
        env_api = EnvironmentApi(zrok)
        custom_headers = {
            'Accept': 'application/json, application/zrok.v1+json'
        }
        resp = env_api.enable_with_http_info(body=req, _headers=custom_headers)

        if hasattr(resp, 'data') and resp.data is not None:
            res = resp.data
        else:
            raise Exception("invalid response from enable API")
    except ApiException as e:
        root.env = Environment()
        if "Unsupported content type: application/zrok.v1+json" in str(e) and hasattr(e, 'body'):
            import json
            try:
                res_dict = json.loads(e.body)

                class EnableResponse:
                    def __init__(self, identity, cfg):
                        self.identity = identity
                        self.cfg = cfg

                res = EnableResponse(
                    identity=res_dict.get('identity', ''),
                    cfg=res_dict.get('cfg', '')
                )
            except (json.JSONDecodeError, ValueError, AttributeError) as json_err:
                raise Exception(f"unable to parse enable response: {str(json_err)}") from e
        else:
            raise Exception("unable to enable environment", e)
    except Exception as e:
        root.env = Environment()
        raise Exception("unable to enable environment", e)

    env = Environment(
        Token=token,
        ZitiIdentity=res.identity,
        ApiEndpoint=api_endpoint.endpoint,
    )

    root.SetEnvironment(env)
    root.SaveZitiIdentityNamed(root.EnvironmentIdentityName(), res.cfg)

    return env


def disable(root: Root) -> None:
    """Disable the current zrok environment.

    Analogous to ``zrok2 disable``. No-op if not currently enabled.

    Args:
        root: The Root environment context.
    """
    if not root.IsEnabled():
        return

    try:
        zrok = root.Client()
    except Exception as e:
        raise Exception("error getting zrok client", e)

    try:
        req = ApiDisableRequest(identity=root.env.ZitiIdentity)
        env_api = EnvironmentApi(zrok)
        custom_headers = {
            'Accept': 'application/json, application/zrok.v1+json'
        }
        env_api.disable_with_http_info(body=req, _headers=custom_headers)
    except ApiException as e:
        if "Unsupported content type: application/zrok.v1+json" in str(e) and (200 <= e.status <= 299):
            pass
        else:
            raise Exception("unable to disable environment", e)
    except Exception as e:
        raise Exception("unable to disable environment", e)

    root.DeleteEnvironment()



from . import environment # noqa
from . import access, listing, model, name, share, overview, status # noqa
try:
    from . import decor  # noqa — requires openziti native SDK
except ImportError:
    pass
from . import _version
__version__ = _version.get_versions()['version']



# coding: utf-8

"""
    zrok

    zrok client access

    The version of the OpenAPI document: 2.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from zrok_api.api.share_api import ShareApi


class TestShareApi(unittest.TestCase):
    """ShareApi unit test stubs"""

    def setUp(self) -> None:
        self.api = ShareApi()

    def tearDown(self) -> None:
        pass

    def test_access(self) -> None:
        """Test case for access

        """
        pass

    def test_create_share_name(self) -> None:
        """Test case for create_share_name

        """
        pass

    def test_delete_share_name(self) -> None:
        """Test case for delete_share_name

        """
        pass

    def test_list_all_names(self) -> None:
        """Test case for list_all_names

        """
        pass

    def test_list_names_for_namespace(self) -> None:
        """Test case for list_names_for_namespace

        """
        pass

    def test_list_share_namespaces(self) -> None:
        """Test case for list_share_namespaces

        """
        pass

    def test_share(self) -> None:
        """Test case for share

        """
        pass

    def test_unaccess(self) -> None:
        """Test case for unaccess

        """
        pass

    def test_unshare(self) -> None:
        """Test case for unshare

        """
        pass

    def test_update_access(self) -> None:
        """Test case for update_access

        """
        pass

    def test_update_share(self) -> None:
        """Test case for update_share

        """
        pass

    def test_update_share_name(self) -> None:
        """Test case for update_share_name

        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    zrok

    zrok client access

    The version of the OpenAPI document: 2.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from zrok_api.models.shares_list import SharesList

class TestSharesList(unittest.TestCase):
    """SharesList unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> SharesList:
        """Test SharesList
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `SharesList`
        """
        model = SharesList()
        if include_optional:
            return SharesList(
                shares = [
                    zrok_api.models.share_summary.shareSummary(
                        share_token = '', 
                        z_id = '', 
                        env_zid = '', 
                        share_mode = '', 
                        backend_mode = '', 
                        frontend_endpoints = [
                            ''
                            ], 
                        target = '', 
                        limited = True, 
                        created_at = 56, 
                        updated_at = 56, )
                    ]
            )
        else:
            return SharesList(
        )
        """

    def testSharesList(self):
        """Test SharesList"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



from zrok2.environment.root import Root
import zrok2.model as model


def status(root: Root) -> model.Status:
    """Return environment status.

    Analogous to ``zrok2 status``.

    Args:
        root: The Root environment context.

    Returns:
        A Status object with environment state information.
    """
    api_endpoint = root.ApiEndpoint()

    return model.Status(
        Enabled=root.IsEnabled(),
        ApiEndpoint=api_endpoint.endpoint,
        ApiEndpointSource=api_endpoint.frm,
        Token=root.env.Token if root.IsEnabled() else "",
        ZitiIdentity=root.env.ZitiIdentity if root.IsEnabled() else "",
    )



from zrok_api.api import ShareApi
from zrok_api.models.create_share_name_request import CreateShareNameRequest
from zrok_api.exceptions import ApiException
from zrok2.environment.root import Root
import zrok2.model as model


_CUSTOM_HEADERS = {
    'Accept': 'application/json, application/zrok.v1+json'
}


def create_name(root: Root, name: str, namespace_token: str = None) -> model.NameEntry:
    """Create a name in a namespace.

    Analogous to ``zrok2 create name <name>``.

    Args:
        root: The Root environment context.
        name: The name to create.
        namespace_token: Optional namespace token. Uses default namespace if omitted.

    Returns:
        A NameEntry with the created name metadata.
    """
    if not root.IsEnabled():
        raise Exception("environment is not enabled; enable with 'zrok enable' first!")

    try:
        zrok = root.Client()
    except Exception as e:
        raise Exception("error getting zrok client", e)

    req = CreateShareNameRequest(
        namespace_token=namespace_token,
        name=name,
    )

    try:
        share_api = ShareApi(zrok)
        share_api.create_share_name_with_http_info(body=req, _headers=_CUSTOM_HEADERS)
    except ApiException as e:
        if "Unsupported content type: application/zrok.v1+json" in str(e) and (200 <= e.status <= 299):
            pass
        else:
            raise Exception(f"unable to create name '{name}'", e)
    except Exception as e:
        raise Exception(f"unable to create name '{name}'", e)

    return model.NameEntry(
        NamespaceToken=namespace_token or "",
        Name=name,
    )


def delete_name(root: Root, name: str, namespace_token: str = None) -> None:
    """Delete a name from a namespace.

    Analogous to ``zrok2 delete name <name>``.

    Args:
        root: The Root environment context.
        name: The name to delete.
        namespace_token: Optional namespace token.
    """
    if not root.IsEnabled():
        raise Exception("environment is not enabled; enable with 'zrok enable' first!")

    try:
        zrok = root.Client()
    except Exception as e:
        raise Exception("error getting zrok client", e)

    req = CreateShareNameRequest(
        namespace_token=namespace_token,
        name=name,
    )

    try:
        share_api = ShareApi(zrok)
        share_api.delete_share_name_with_http_info(body=req, _headers=_CUSTOM_HEADERS)
    except ApiException as e:
        if "Unsupported content type: application/zrok.v1+json" in str(e) and (200 <= e.status <= 299):
            pass
        else:
            raise Exception(f"unable to delete name '{name}'", e)
    except Exception as e:
        raise Exception(f"unable to delete name '{name}'", e)


def list_names(root: Root, namespace_token: str = None) -> list[model.NameEntry]:
    """List names in a namespace.

    Analogous to ``zrok2 list names``.

    Args:
        root: The Root environment context.
        namespace_token: If provided, list names only in this namespace.

    Returns:
        List of NameEntry objects.
    """
    if not root.IsEnabled():
        raise Exception("environment is not enabled; enable with 'zrok enable' first!")

    try:
        zrok = root.Client()
    except Exception as e:
        raise Exception("error getting zrok client", e)

    try:
        share_api = ShareApi(zrok)
        if namespace_token:
            res = share_api.list_names_for_namespace(namespace_token=namespace_token)
        else:
            # list_share_namespaces returns namespaces; for all names we iterate
            namespaces = share_api.list_share_namespaces()
            res = []
            for ns in namespaces:
                ns_names = share_api.list_names_for_namespace(namespace_token=ns.namespace_token)
                res.extend(ns_names)
    except Exception as e:
        raise Exception("unable to list names", e)

    return [
        model.NameEntry(
            NamespaceToken=n.namespace_token or "",
            NamespaceName=n.namespace_name or "",
            Name=n.name or "",
            ShareToken=n.share_token or "",
            Reserved=n.reserved or False,
            CreatedAt=n.created_at or 0,
        )
        for n in res
    ]


def list_namespaces(root: Root) -> list[model.Namespace]:
    """List all available namespaces.

    Analogous to ``zrok2 list namespaces``.

    Args:
        root: The Root environment context.

    Returns:
        List of Namespace objects.
    """
    if not root.IsEnabled():
        raise Exception("environment is not enabled; enable with 'zrok enable' first!")

    try:
        zrok = root.Client()
    except Exception as e:
        raise Exception("error getting zrok client", e)

    try:
        share_api = ShareApi(zrok)
        res = share_api.list_share_namespaces()
    except Exception as e:
        raise Exception("unable to list namespaces", e)

    return [
        model.Namespace(
            NamespaceToken=ns.namespace_token or "",
            Name=ns.name or "",
            Description=ns.description or "",
        )
        for ns in res
    ]



from setuptools import setup, find_packages  # noqa: H301
import os
import versioneer

NAME = os.getenv('ZROK_PY_NAME') or "zrok2"
VERSION = "2.0.0"

REQUIRES = [
    "openziti >= 1.0.0",
    "urllib3 >= 2.1.0",
    "pydantic >= 2",
    "python_dateutil >= 2.8.2",
    "typing-extensions >= 4.7.1",
]

setup(
    name=NAME,
    cmdclass=versioneer.get_cmdclass(dict()),
    version=versioneer.get_version(),
    description="zrok2",
    author_email="",
    url="",
    keywords=["zrok", "zrok2"],
    install_requires=REQUIRES,
    extras_require={
        "test": ["pytest>=7.0", "pytest-cov"],
    },
    python_requires='>3.10.0',
    packages=find_packages(),
    include_package_data=True,
    package_data={
        '': ['requirements.txt'],  # Include the generated requirements.txt in the package
    },
    long_description="""\
    Geo-scale, next-generation peer-to-peer sharing platform built on top of OpenZiti.
    """
)



from zrok_api.api import MetadataApi
from zrok2.environment.root import Root
import zrok2.model as model


def list_shares(root: Root, **filters) -> list[model.ShareDetail]:
    """List shares with optional filters.

    Analogous to ``zrok2 list shares``.

    Args:
        root: The Root environment context.
        **filters: Optional keyword filters passed to the API:
            env_zid, share_mode, backend_mode, share_token, target,
            permission_mode, has_activity, idle, activity_duration,
            created_after, created_before, updated_after, updated_before.

    Returns:
        List of ShareDetail objects.
    """
    if not root.IsEnabled():
        raise Exception("environment is not enabled; enable with 'zrok enable' first!")

    try:
        zrok = root.Client()
    except Exception as e:
        raise Exception("error getting zrok client", e)

    try:
        metadata_api = MetadataApi(zrok)
        res = metadata_api.list_shares(**filters)
    except Exception as e:
        raise Exception("unable to list shares", e)

    shares = res.shares or []
    return [
        model.ShareDetail(
            Token=s.share_token or "",
            ZId=s.z_id or "",
            EnvZId=s.env_zid or "",
            ShareMode=s.share_mode or "",
            BackendMode=s.backend_mode or "",
            FrontendEndpoints=s.frontend_endpoints or [],
            Target=s.target or "",
            Limited=s.limited or False,
            CreatedAt=s.created_at or 0,
            UpdatedAt=s.updated_at or 0,
        )
        for s in shares
    ]


def list_accesses(root: Root, **filters) -> list[model.AccessDetail]:
    """List accesses with optional filters.

    Analogous to ``zrok2 list accesses``.

    Args:
        root: The Root environment context.
        **filters: Optional keyword filters passed to the API:
            env_zid, share_token, bind_address, description,
            created_after, created_before, updated_after, updated_before.

    Returns:
        List of AccessDetail objects.
    """
    if not root.IsEnabled():
        raise Exception("environment is not enabled; enable with 'zrok enable' first!")

    try:
        zrok = root.Client()
    except Exception as e:
        raise Exception("error getting zrok client", e)

    try:
        metadata_api = MetadataApi(zrok)
        res = metadata_api.list_accesses(**filters)
    except Exception as e:
        raise Exception("unable to list accesses", e)

    accesses = res.accesses or []
    return [
        model.AccessDetail(
            Id=a.id or 0,
            FrontendToken=a.frontend_token or "",
            EnvZId=a.env_zid or "",
            ShareToken=a.share_token or "",
            BackendMode=a.backend_mode or "",
            BindAddress=a.bind_address or "",
            Description=a.description or "",
            Limited=a.limited or False,
            CreatedAt=a.created_at or 0,
            UpdatedAt=a.updated_at or 0,
        )
        for a in accesses
    ]



# coding: utf-8

"""
    zrok

    zrok client access

    The version of the OpenAPI document: 2.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing import Optional, Set
from typing_extensions import Self

class AccessSummary(BaseModel):
    """
    AccessSummary
    """ # noqa: E501
    id: Optional[StrictInt] = None
    frontend_token: Optional[StrictStr] = Field(default=None, alias="frontendToken")
    env_zid: Optional[StrictStr] = Field(default=None, alias="envZId")
    share_token: Optional[StrictStr] = Field(default=None, alias="shareToken")
    backend_mode: Optional[StrictStr] = Field(default=None, alias="backendMode")
    bind_address: Optional[StrictStr] = Field(default=None, alias="bindAddress")
    description: Optional[StrictStr] = None
    limited: Optional[StrictBool] = None
    created_at: Optional[StrictInt] = Field(default=None, alias="createdAt")
    updated_at: Optional[StrictInt] = Field(default=None, alias="updatedAt")
    __properties: ClassVar[List[str]] = ["id", "frontendToken", "envZId", "shareToken", "backendMode", "bindAddress", "description", "limited", "createdAt", "updatedAt"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of AccessSummary from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of AccessSummary from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id": obj.get("id"),
            "frontendToken": obj.get("frontendToken"),
            "envZId": obj.get("envZId"),
            "shareToken": obj.get("shareToken"),
            "backendMode": obj.get("backendMode"),
            "bindAddress": obj.get("bindAddress"),
            "description": obj.get("description"),
            "limited": obj.get("limited"),
            "createdAt": obj.get("createdAt"),
            "updatedAt": obj.get("updatedAt")
        })
        return _obj


